/**
 * 生成babelrc配置文件，并更新.babelrc，供依赖 babel-loader 的非js文件解析使用
 */
let path = require('path');

let fs = require('fs');

let browsers = require('./browsers');

const defaultPlugins = [require.resolve("@babel/plugin-syntax-dynamic-import"), // 动态import
require.resolve("@babel/plugin-syntax-import-meta"), require.resolve("@babel/plugin-transform-modules-commonjs"), // transforms ES2015 modules to CommonJS
[require.resolve("@babel/plugin-transform-runtime"), {
  // 运行时帮助函数库
  "corejs": 3
}]];

const babelrc = (config, isDebug) => {
  let {
    presets = [],
    plugins: userConfigPlugins = []
  } = config.babelrc || {};
  return {
    presets: [[require.resolve("@babel/preset-env"), {
      "targets": {
        // https://github.com/ai/browserslist http://browserl.ist/
        "browsers": browsers.getBrowserList(config, isDebug)
      },
      // 开启modules=false可以使用webpack2的treeshaking，但是，会不兼容amd cmd等模块
      // './lib/kEvent', './lib/react-tabs-redux-custom', './apps/utils/global'
      "modules": false,
      "useBuiltIns": "usage",
      "corejs": 3,
      "debug": false
    }]].concat(presets),
    plugins: defaultPlugins.concat(userConfigPlugins)
  };
};

const bablerc4Server = config => {
  let {
    presets = [],
    plugins: userConfigPlugins = []
  } = config.babelrc || {};
  return {
    presets: [[require.resolve("@babel/preset-env"), {
      "targets": {
        "node": "current"
      },
      "useBuiltIns": "usage",
      "corejs": 3
    }]].concat(presets),
    plugins: defaultPlugins.concat(userConfigPlugins)
  };
};

const update = (config, isDebug) => {
  let babelpath = path.resolve(process.cwd(), './.babelrc');
  let newBabelrc = JSON.stringify(babelrc(config, isDebug), null, 4);

  if (!fs.existsSync(babelpath) || fs.readFileSync(babelpath, 'utf8') !== newBabelrc) {
    fs.writeFileSync(babelpath, newBabelrc);
  }
};

module.exports = {
  update,

  /**
   * babel-loader参数容易配置
   */
  dllOptions(config) {
    return {
      // https://github.com/babel/babel-loader#options
      cacheDirectory: true,
      // https://babeljs.io/docs/usage/options/
      babelrc: false,
      ...babelrc(config, false)
    };
  },

  /**
   * babel-loader参数容易配置
   */
  clientOptions(config, DEBUG) {
    return {
      // https://github.com/babel/babel-loader#options
      cacheDirectory: true,
      // https://babeljs.io/docs/usage/options/
      babelrc: false,
      ...babelrc(config, DEBUG)
    };
  },

  /**
   * webpack-server-config用于node端的 babelrc
   */
  serverOptions(config) {
    return {
      // https://github.com/babel/babel-loader#options
      cacheDirectory: true,
      // https://babeljs.io/docs/usage/options/
      babelrc: false,
      ...bablerc4Server(config)
    };
  }

};