import * as babelrc from './babelrc';
/**
* babel7 loader配置
*
* @param {*} chainedConfig  webpack 配置
* @param {*} rulename rule 的名称
* @param {*} option babel选项
*/

const babelLoader = (chainedConfig, rulename = 'js-compile', option) => {
  chainedConfig.module.rule(rulename).use('babel-loader').loader(require.resolve('babel-loader')).options(option);
};

export const options = joi => {
  return {
    pluginScheme: {
      babelrc: joi.object({
        include: joi.array().items(joi.string()),
        plugins: joi.alternatives(joi.array(), joi.string()),
        presets: joi.alternatives(joi.array(), joi.string())
      }),
      dll: joi.object({
        // 同时使用 polyfill 和配置含有es6的实例方法的chunk，可能会出问题。例如：  Cannot read property 'includes' of undefined
        // 因此，尽量不要直接使用项目中的文件生成dll
        // usePolyfill 是false的时候，可以使用项目中的文件，不会产生问题
        chunks: joi.object(),
        usePolyfill: joi.boolean()
      })
    },
    pluginDefaultOptions: {
      /**
       * {
       *    presets: {Array<string> | string}
       *    plugins: {Array<string> | string}
       *  }
       */
      babelrc: {// 可以使用以下配置尝试新功能
        // plugins:[
        //     "@babel/plugin-proposal-class-properties",
        //     ["@babel/plugin-proposal-decorators", {legacy: true}],
        //     "@babel/plugin-proposal-do-expressions",
        //     "@babel/plugin-proposal-export-default-from",
        //     "@babel/plugin-proposal-export-namespace-from",
        //     "@babel/plugin-proposal-function-bind",
        //     "@babel/plugin-proposal-function-sent",
        //     "@babel/plugin-proposal-json-strings",
        //     "@babel/plugin-proposal-logical-assignment-operators",
        //     "@babel/plugin-proposal-nullish-coalescing-operator",
        //     "@babel/plugin-proposal-numeric-separator",
        //     "@babel/plugin-proposal-optional-chaining",
        //     ["@babel/plugin-proposal-pipeline-operator", { "proposal": "minimal" }],
        //     "@babel/plugin-proposal-throw-expressions",
        // ]
      },
      dll: {
        chunks: {},
        usePolyfill: true
      }
    }
  };
};
export const clientConf = (chainedConfig, env, config) => {
  if (config.dll.usePolyfill) {
    chainedConfig.entryPoints.values().forEach(entry => {
      entry.prepend('regenerator-runtime/runtime');
      entry.prepend('core-js/stable'); //加载项目中安装的core-js，逐级查找
    });
  }

  babelLoader(chainedConfig, 'js-compile', babelrc.clientOptions(config, env === 'DEBUG'));
};
export const dllConf = (chainedConfig, env, config) => {
  //处理旧配置文件
  if (config.dllchunk) {
    config.dll.usePolyfill = true;
  }

  if (config.dll.usePolyfill) {
    chainedConfig.entry('tua__polyfill').add('core-js/stable'); //加载项目中安装的core-js，逐级查找

    chainedConfig.entry('tua__polyfill__pure').add('core-js-pure') //加载项目中安装的core-js-pure，逐级查找
    .add('regenerator-runtime/runtime');
  }

  babelLoader(chainedConfig, 'js-compile', babelrc.dllOptions(config));
};
export const serverConf = (chainedConfig, env, config) => {
  babelLoader(chainedConfig, 'js-compile', babelrc.serverOptions(config));

  if (env !== 'DEBUG') {
    babelLoader(chainedConfig, 'insertMfs', babelrc.serverOptions(config));
  }
};
export const beforeCompile = (env, config, options) => {
  babelrc.update(config, env === 'DEBUG');
};
export const generate = options => {
  return {
    "dependencies": {
      "core-js": "^3.1.4",
      "@babel/runtime-corejs3": "^7.5.2"
    }
  };
};